"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDevServer = exports.devMiddleware = exports.deviceMiddleware = void 0;
const fs_1 = __importDefault(require("fs"));
const express_1 = __importDefault(require("express"));
const path_1 = __importDefault(require("path"));
const resolveDevice_1 = require("./resolveDevice");
const DEVICE_MAP = {
    mobile: /Mobile|iP(hone|od|ad)|Android|BlackBerry|IEMobile|Kindle|NetFront|Silk-Accelerated|(hpw|web)OS|Fennec|Minimo|Opera M(obi|ini)|Blazer|Dolfin|Dolphin|Skyfire|Zune/,
    desktop: true,
};
const deviceMiddleware = (devicesMap) => (req, res, next) => {
    req.device = resolveDevice_1.resolveDevice(req, devicesMap);
    next();
};
exports.deviceMiddleware = deviceMiddleware;
function devMiddleware(root = null, deviceMap = DEVICE_MAP) {
    const { createServer: createViteServer } = require('vite');
    root = root || process.cwd();
    const vite = {};
    // const serverReady: Promise<unknown> =
    (async () => {
        for (const device of Object.keys(deviceMap)) {
            vite[device] = await createViteServer({
                root,
                logLevel: 'info',
                server: {
                    middlewareMode: true,
                },
                cacheDir: 'node_modules/.vite/' + device,
                __DEVICE: device,
            });
        }
    })();
    const template = fs_1.default.readFileSync('./index.html', 'utf-8');
    return [
        exports.deviceMiddleware(deviceMap),
        (req, res, next) => {
            if (!req.device) {
                return res.end('Device could not be guessed. check your configurations.');
            }
            if (!vite[req.device]) {
                res.writeHead(503, { 'Retry-After': '5' });
                return res.end(fs_1.default.readFileSync(path_1.default.resolve(__dirname, '../loading.html'), 'utf-8'));
            }
            vite[req.device].middlewares.handle(req, res, next);
        },
        async (req, res) => {
            const url = req.originalUrl;
            // always read fresh template in dev
            const html = await vite[req.device].transformIndexHtml(url, template);
            res.writeHead(200, { 'Content-Type': 'text/html' });
            res.end(html);
        }
    ];
}
exports.devMiddleware = devMiddleware;
function createDevServer(root = null, deviceMap = DEVICE_MAP) {
    const app = express_1.default();
    app.use(devMiddleware(root, deviceMap));
    return app;
}
exports.createDevServer = createDevServer;
